<?php

/**
 * PHP
 * 2021-02-03: 
 * ApolloAPI v1.2 (current)  connects to and supports the methods found in version 1.2 of our API using 
 * PHP 5+ Provided by Applicom.
 */

class ApolloAPI {
    var $version = "v1.2";
    var $endpoint = "v1";
    var $errored = false;
    var $errorMessage;
	var $responseStatus;
    
    /**
     * Cache the information on the API location on the server
     */
    var $apiHost = "api.apollohq.com";
    
    /**
     * Default to a 300 second timeout on server calls
     */
    var $timeout = 300; 
    
    /**
     * Default to a 8K chunk size
     */
    var $chunkSize = 8192;
    
    /**
     * Cache the user api_key so we only have to log in once per client instantiation
     */
    var $api_key;
    
    /**
     * Connect to the Apollo API for a given list.
     * 
     * @param string $apikey Your Apollo apikey
     * @param string $host API location on the server
     */
	public function __construct($apikey, $host = NULL)
	{
		if($host) {
			$this->apiHost = $host;
		}
        
        $this->api_key = $apikey;
	}
	
    public function setTimeout($seconds){
        if (is_int($seconds)){
            $this->timeout = $seconds;
            return true;
        }
    }
	
    public function getTimeout(){
        return $this->timeout;
    }
    
    /**
     * Actually connect to the server and call the requested methods, parsing the result
     * You should never have to call this function manually
     */
    private function callServer($method, $url, $params = array()) {
		$this->errorMessage = "";
      
        $post_vars = "";
		$get_vars = "";
		if(strtoupper(trim($method)) === "GET") {
			$get_vars = "?".http_build_query($params, '', '&');
		}
		else {
			$post_vars = json_encode($params);
		}
        
        $payload = "$method /". $this->endpoint ."$url$get_vars HTTP/1.0\r\n";
        $payload .= "Host: " . $this->apiHost . "\r\n";
        $payload .= "Authorization: Basic " . base64_encode($this->api_key . ":X") . "\r\n";
        $payload .= "User-Agent: ApolloAPI/" . $this->version ."\r\n";
        $payload .= "Content-type: application/json\r\n";
		$payload .= "Content-length: " . strlen($post_vars) . "\r\n";
		$payload .= "Connection: close \r\n\r\n";
		$payload .= $post_vars;
		
        ob_start();
        $sock = fsockopen("ssl://".$this->apiHost, 443, $errno, $errstr, 30);
        if(!$sock) {
            $this->errorMessage = "Could not connect (ERR $errno: $errstr)";
            ob_end_clean();
            return false;
        }
        
        $response = "";
        fwrite($sock, $payload);
        stream_set_timeout($sock, $this->timeout);
        $info = stream_get_meta_data($sock);
        while ((!feof($sock)) && (!$info["timed_out"])) {
            $response .= fread($sock, $this->chunkSize);
            $info = stream_get_meta_data($sock);
        }
        fclose($sock);
        ob_end_clean();
        if ($info["timed_out"]) {
            $this->errorMessage = "Could not read response (timed out)";
            return false;
        }
	
        list($headers, $response) = explode("\r\n\r\n", $response, 2);
        $headers = explode("\r\n", $headers);
		$this->errored = false;
		foreach($headers as $h){
            if(preg_match("/^HTTP\/1\.1 /", $h, $matches) > 0) {
                $this->responseStatus = substr($h, 9, 3);
                break;
            }
        }
		if(substr($this->responseStatus, 0, 1) != 2) {
			$this->errored = true;
		}
        
        if($this->errored) {
            $this->errorMessage = $response;
            return false;
        }
        
        return $response;
    }
	
	/**
     * Support function
     * You should never have to call this function manually
     */
	private function adjustParameters($parameters, $entity_name) {
		return array($entity_name => $parameters);
	}
	
	/**
     * Returns all accessible projects. This includes active, silent, template and archived projects
     *
     * @section Projects
     * @docs http://apidocs.apollohq.com/projects
     * @example getProjects.php
     *
     * @return list of projects
     */
    public function getProjects() {
        return $this->callServer("GET", "/projects");
    }
	
	/**
     * Returns a count of all projects, by project status
     *
     * @section Projects
	 * @docs http://apidocs.apollohq.com/projects
     * @example countProjects.php
     *
     * @return count of all projects, by project status
     */
    public function countProjects() {
        return $this->callServer("GET", "/projects/count");
    }
	
	/**
     * Returns a single project identified by its integer ID
     *
     * @section Projects
	 * @docs http://apidocs.apollohq.com/projects
     * @example getProject.php
     *
	 * @param integer $project_id the ID of the project
     * @return project data
     */
    public function getProject($project_id) {
        return $this->callServer("GET", "/projects/$project_id");
    }
	
	/**
     * Creates a new project with the given name. Only administrative users may create projects
     *
     * @section Projects
	 * @docs http://apidocs.apollohq.com/projects
     * @example createProject.php
     *
	 * @param array $project_data an associative array with the project data
     * @return new project id
     */
    public function createProject($project_data) {
		$params = $this->adjustParameters($project_data, "project");
        return $this->callServer("POST", "/projects", $params);
    }
	
	/**
     * Updates attributes of the given project. Only administrative users may update project records.
     *
     * @section Projects
	 * @docs http://apidocs.apollohq.com/projects
     * @example updateProject.php
     *
	 * @param integer $project_id the project id
	 * @param array $project_data an associative array with the project data
     */
    public function updateProject($project_id, $project_data) {
		$params = $this->adjustParameters($project_data, "project");
        return $this->callServer("PUT", "/projects/$project_id", $params);
    }
	
	/**
     * Returns the 25 most recent messages in the given project. If there are older messages not included in the 
	 * response, the root response element will also have a 'continued-at' attribute specifying the path where 
	 * the next oldest 25 messages can be retrieved.
     *
     * @section Messages
     * @docs http://apidocs.apollohq.com/messages
     * @example getMessages.php
     *
     * @return list of messages
     */
    public function getMessages($project_id, $page = 1) {
        return $this->callServer("GET", "/projects/$project_id/messages/$page");
    }
	
	/**
     * Returns a single message identified by its integer ID
     *
     * @section Messages
	 * @docs http://apidocs.apollohq.com/messages
     * @example getMessage.php
     *
	 * @param integer $message_id the ID of the message
     * @return message data
     */
    public function getMessage($message_id) {
        return $this->callServer("GET", "/messages/$message_id");
    }
	
	/**
     * Returns the 25 most recent messages in the given project for the given category. If there are older messages 
	 * not included in the response, the root response element will also have a 'continued-at' attribute specifying 
	 * the path where the next oldest 25 messages can be retrieved.
     *
     * @section Messages
     * @docs http://apidocs.apollohq.com/messages
     * @example getMessagesByCategory.php
     *
     * @return list of messages
     */
    public function getMessagesByCategory($project_id, $category_id, $page = 1) {
        return $this->callServer("GET", "/projects/$project_id/cat/$category_id/messages/$page");
    }
	
	/**
     * Creates a new message, optionally sending notifications to a selected list of people. Note that you can also 
	 * upload files using this function, but you need to upload the files first and then attach them.
     *
     * @section Messages
	 * @docs http://apidocs.apollohq.com/messages
     * @example createMessage.php
     *
	 * @param integer $project_id the id of the project
	 * @param array $message_data an associative array with the message data
     * @return new message id
     */
    public function createMessage($project_id, $message_data) {
		$params = $this->adjustParameters($message_data, "message");
        return $this->callServer("POST", "/projects/$project_id/messages", $params);
    }
	
	/**
     * Updates an existing message, optionally sending notifications to a selected list of people. Note that 
	 * you can also upload files using this function, but you need to upload the files first and then attach them.
     *
     * @section Messages
	 * @docs http://apidocs.apollohq.com/messages
     * @example updateMessage.php
     *
	 * @param integer $message_id the message id
	 * @param array $message_data an associative array with the message data
     */
    public function updateMessage($message_id, $message_data) {
		$params = $this->adjustParameters($message_data, "message");
        return $this->callServer("PUT", "/messages/$message_id", $params);
    }
	
	/**
     * Destroys the given message and all of its associated comments. 
     *
     * @section Messages
	 * @docs http://apidocs.apollohq.com/messages
     * @example deleteMessage.php
     *
	 * @param integer $message_id the message id
     */
    public function deleteMessage($message_id) {
        return $this->callServer("DELETE", "/messages/$message_id");
    }
	
	/**
     * Returns all task records for a single task list.
     *
     * @section Tasks
     * @docs http://apidocs.apollohq.com/tasks
     * @example getTasksByTaskList.php
     *
     * @return list of tasks
     */
    public function getTasksByTaskList($task_list_id) {
        return $this->callServer("GET", "/taskLists/$task_list_id/tasks");
    }
	
	/**
     * Returns all task records for a project inbox.
     *
     * @section Tasks
     * @docs http://apidocs.apollohq.com/tasks
     * @example getInboxTasks.php
     *
     * @return list of tasks
     */
    public function getInboxTasks($project_id) {
        return $this->callServer("GET", "/projects/$project_id/inbox");
    }
	
	/**
     * Returns all task records for a contact.
     *
     * @section Tasks
     * @docs http://apidocs.apollohq.com/tasks
     * @example getTasksByContact.php
     *
     * @return list of tasks
     */
    public function getTasksByContact($contact_id) {
        return $this->callServer("GET", "/contacts/$contact_id/tasks");
    }
	
	/**
     * Returns all task records for a case.
     *
     * @section Tasks
     * @docs http://apidocs.apollohq.com/tasks
     * @example getTasksByCase.php
     *
     * @return list of tasks
     */
    public function getTasksByCase($case_id) {
        return $this->callServer("GET", "/cases/$case_id/tasks");
    }
	
	/**
     * Returns all task records for a deal.
     *
     * @section Tasks
     * @docs http://apidocs.apollohq.com/tasks
     * @example getTasksByDeal.php
     *
     * @return list of tasks
     */
    public function getTasksByDeal($deal_id) {
        return $this->callServer("GET", "/deals/$deal_id/tasks");
    }
	
	/**
     * Returns the personal tasks that are visible to the authenticated user. Use the search query to refine your 
	 * search. Available parameters are:
     *  - "page": if the results are many you can jump to the next page using the "page" parameter.
     *  - "status": possible values are 'started', 'notstarted' or 'done' (if you don't pass nothing both 'started' 
	 *    and 'notstarted' are returned).
     *
     * @section Tasks
     * @docs http://apidocs.apollohq.com/tasks
     * @example getPersonalTasks.php
     *
     * @return list of tasks
     */
    public function getPersonalTasks() {
        return $this->callServer("GET", "/tasks/personal");
    }
	
	/**
     * Returns a single task item identified by its integer ID
     *
     * @section Tasks
	 * @docs http://apidocs.apollohq.com/tasks
     * @example getTask.php
     *
	 * @param integer $task_id the ID of the task
     * @return task data
     */
    public function getTask($task_id) {
        return $this->callServer("GET", "/tasks/$task_id");
    }
	
	/**
     * Marks the specified task item as done.
     *
     * @section Tasks
	 * @docs http://apidocs.apollohq.com/tasks
     * @example completeTask.php
     *
	 * @param integer $task_id the ID of the task
     */
    public function completeTask($task_id) {
        return $this->callServer("PUT", "/tasks/$task_id/done");
    }
	
	/**
     * Marks the specified task item as started.
     *
     * @section Tasks
	 * @docs http://apidocs.apollohq.com/tasks
     * @example startTask.php
     *
	 * @param integer $task_id the ID of the task
     */
    public function startTask($task_id) {
        return $this->callServer("PUT", "/tasks/$task_id/started");
    }
	
	/**
     * Marks the specified task item as not started.
     *
     * @section Tasks
	 * @docs http://apidocs.apollohq.com/tasks
     * @example resetTask.php
     *
	 * @param integer $task_id the ID of the task
     */
    public function resetTask($task_id) {
        return $this->callServer("PUT", "/tasks/$task_id/notstarted");
    }
	
	/**
     * Creates a new task record for the given list. The new record begins its life in the "not started" state. 
	 * It is added at the bottom of the given list. If a person or a company is responsible for the task, give 
	 * their id as the responsible-id value. If your current user is not the task's responsible, you can also use 
	 * the "notify" key to indicate whether an email should be sent to you to tell you about the task's completion. 
	 * You can specify the task due date with the "due-date" key (containing an UTC timestamp) or with the 
	 * "time-bucket" key. Possible value for it are: "later" (if you don't want to set a due date), "today", 
	 * "tomorrow", "this week" and "next week".
     *
     * @section Tasks
	 * @docs http://apidocs.apollohq.com/tasks
     * @example createTaskInTaskList.php
     *
	 * @param integer $task_list_id the id of the task list
	 * @param array $task_data an associative array with the task data
     * @return new task id
     */
    public function createTaskInTaskList($task_list_id, $task_data) {
		$params = $this->adjustParameters($task_data, "task");
        return $this->callServer("POST", "/taskLists/$task_list_id/tasks", $params);
    }
	
	/**
     * Creates a new task record for the given project inbox. The new record begins its life in the "not started" state. 
	 * It is added at the bottom of the given list. If a person or a company is responsible for the task, give 
	 * their id as the responsible-id value. If your current user is not the task's responsible, you can also use 
	 * the "notify" key to indicate whether an email should be sent to you to tell you about the task's completion. 
	 * You can specify the task due date with the "due-date" key (containing an UTC timestamp) or with the 
	 * "time-bucket" key. Possible value for it are: "later" (if you don't want to set a due date), "today", 
	 * "tomorrow", "this week" and "next week".
     *
     * @section Tasks
	 * @docs http://apidocs.apollohq.com/tasks
     * @example createTaskInInbox.php
     *
	 * @param integer $project_id the id of the project
	 * @param array $task_data an associative array with the task data
     * @return new task id
     */
    public function createTaskInInbox($project_id, $task_data) {
		$params = $this->adjustParameters($task_data, "task");
        return $this->callServer("POST", "/projects/$project_id/inbox", $params);
    }
	
	/**
     * Creates a new task record for the given contact. The new record begins its life in the "not started" state. 
	 * It is added at the bottom of the list of task. A person or a company must be responsible for the task, give 
	 * their id as the responsible-id value. If your current user is not the task's responsible, you can also use 
	 * the "notify" key to indicate whether an email should be sent to you to tell you about the task's completion. 
	 * You can specify the task due date with the "due-date" key (containing an UTC timestamp) or with the 
	 * "time-bucket" key. Possible value for it are: "later" (if you don't want to set a due date), "today", 
	 * "tomorrow", "this week" and "next week".
     *
     * @section Tasks
	 * @docs http://apidocs.apollohq.com/tasks
     * @example createContactTask.php
     *
	 * @param integer $contact_id the id of the contact
	 * @param array $task_data an associative array with the task data
     * @return new task id
     */
    public function createContactTask($contact_id, $task_data) {
		$params = $this->adjustParameters($task_data, "task");
        return $this->callServer("POST", "/contacts/$contact_id/tasks", $params);
    }
	
	/**
     * Creates a new task record for the given case. The new record begins its life in the "not started" state. 
	 * It is added at the bottom of the list of task. A person or a company must be responsible for the task, give 
	 * their id as the responsible-id value. If your current user is not the task's responsible, you can also use 
	 * the "notify" key to indicate whether an email should be sent to you to tell you about the task's completion. 
	 * You can specify the task due date with the "due-date" key (containing an UTC timestamp) or with the 
	 * "time-bucket" key. Possible value for it are: "later" (if you don't want to set a due date), "today", 
	 * "tomorrow", "this week" and "next week".
     *
     * @section Tasks
	 * @docs http://apidocs.apollohq.com/tasks
     * @example createCaseTask.php
     *
	 * @param integer $case_id the id of the case
	 * @param array $task_data an associative array with the task data
     * @return new task id
     */
    public function createCaseTask($case_id, $task_data) {
		$params = $this->adjustParameters($task_data, "task");
        return $this->callServer("POST", "/cases/$case_id/tasks", $params);
    }
	
	/**
     * Creates a new task record for the given deal. The new record begins its life in the "not started" state. 
	 * It is added at the bottom of the list of task. A person or a company must be responsible for the task, give 
	 * their id as the responsible-id value. If your current user is not the task's responsible, you can also use 
	 * the "notify" key to indicate whether an email should be sent to you to tell you about the task's completion. 
	 * You can specify the task due date with the "due-date" key (containing an UTC timestamp) or with the 
	 * "time-bucket" key. Possible value for it are: "later" (if you don't want to set a due date), "today", 
	 * "tomorrow", "this week" and "next week".
     *
     * @section Tasks
	 * @docs http://apidocs.apollohq.com/tasks
     * @example createDealTask.php
     *
	 * @param integer $deal_id the id of the deal
	 * @param array $task_data an associative array with the task data
     * @return new task id
     */
    public function createDealTask($deal_id, $task_data) {
		$params = $this->adjustParameters($task_data, "task");
        return $this->callServer("POST", "/deals/$deal_id/tasks", $params);
    }
	
	/**
     * Creates a new personal task record. The new record begins its life in the "not started"
	 * state. (See the "Complete", "Mark as started"; and "Mark as not started" actions.) 
	 * Give the id of the person responsible for the task as the responsible-id value. If your current user is not the 
	 * task's responsible, you can also use the "notify" key to indicate whether an email should be sent to 
	 * you to tell you about the task's completion. You can specify the task due date with the "due-date" key 
	 * (containing an UTC timestamp) or with the "time-bucket" key. Possible value for it are: 
	 * "later" (if you don't want to set a due date), "today", "tomorrow", 
	 * "this week" and "next week".
     *
     * @section Tasks
	 * @docs http://apidocs.apollohq.com/tasks
     * @example createPersonalTask.php
     *
	 * @param array $task_data an associative array with the task data
     * @return new task id
     */
    public function createPersonalTask($task_data) {
		$params = $this->adjustParameters($task_data, "task");
        return $this->callServer("POST", "/tasks/personal", $params);
    }
	
	/**
     * Updates a specific task.
     *
     * @section Tasks
	 * @docs http://apidocs.apollohq.com/tasks
     * @example updateTask.php
     *
	 * @param integer $task_id the task id
	 * @param array $task_data an associative array with the task data
     */
    public function updateTask($task_id, $task_data) {
		$params = $this->adjustParameters($task_data, "task");
        return $this->callServer("PUT", "/tasks/$task_id", $params);
    }
	
	/**
     * Delete the task with the given ID. 
     *
     * @section Tasks
	 * @docs http://apidocs.apollohq.com/tasks
     * @example deleteTask.php
     *
	 * @param integer $task_id the task id
     */
    public function deleteTask($task_id) {
        return $this->callServer("DELETE", "/tasks/$task_id");
    }
	
	/**
     * Changes the ordering of the tasks for the given list. Completed tasks cannot be reordered, and any task not 
	 * specified will be sorted after the tasks explicitly given (allowing you to easily move a single task to the 
	 * head of the list without having to specify the positions of all the other tasks). You may reparent tasks by 
	 * putting tasks from one list into the ordering of tasks for a different list (though tasks cannot be 
	 * reparented across project boundaries). 
     *
     * @section Tasks
	 * @docs http://apidocs.apollohq.com/tasks
     * @example reorderTasksInTaskList.php
     *
	 * @param integer $task_list_id the task list id
	 * @param array $tasks an array with the ids of the tasks that you want to reorder
     */
    public function reorderTasksInTaskList($task_list_id, $tasks) {
        $params = $this->adjustParameters($tasks, "tasks");
		return $this->callServer("POST", "/taskLists/$task_list_id/tasks/reorder", $params);
    }
	
	/**
     * Changes the ordering of the tasks for the inbox of the given project. 
     *
     * @section Tasks
	 * @docs http://apidocs.apollohq.com/tasks
     * @example reorderTasksInInbox.php
     *
	 * @param integer $project_id the project id
	 * @param array $tasks an array with the ids of the tasks that you want to reorder
     */
    public function reorderTasksInInbox($project_id, $tasks) {
        $params = $this->adjustParameters($tasks, "tasks");
		return $this->callServer("POST", "/projects/$project_id/reorderTasks", $params);
    }
	
	/**
     * Returns a list of task list records, with task records that are assigned to the given "responsible". If no 
	 * responsible is given, the current user is assumed to be the responsible. The responsible may be changed by 
	 * setting the "responsible" parameter to a blank string (for unassigned items), a person-id, or a company-id.
     *
     * @section TaskLists
     * @docs http://apidocs.apollohq.com/taskLists
     * @example getTaskListsByResponsible.php
     *
     * @return list of task lists
     */
    public function getTaskListsByResponsible($responsible = NULL) {
        return $this->callServer("GET", "/taskLists".($responsible !== NULL ? "?responsible=$responsible" : ""));
    }
	
	/**
     * Returns a list of task list records that are in the given project. By default, all lists are returned, but 
	 * you can filter the result by giving the "filter" query parameter, set to "all" (the default), "pending" 
	 * (for lists with uncompleted items), and "finished" (for lists that have no uncompleted items). The lists 
	 * will be returned in priority order, as determined by their ordering. (See the "reorder lists" action.)
     *
     * @section TaskLists
     * @docs http://apidocs.apollohq.com/taskLists
     * @example getTaskListsByProject.php
     *
     * @return list of task lists
     */
    public function getTaskListsByProject($project_id, $filter = NULL) {
        return $this->callServer("GET", "/projects/$project_id/taskLists".($filter !== NULL ? "/$filter" : ""));
    }
	
	/**
     * Returns a single task list record identified by its integer ID.
     *
     * @section TaskLists
     * @docs http://apidocs.apollohq.com/taskLists
     * @example getTaskList.php
     *
     * @return task list data
     */
    public function getTaskList($task_list_id) {
        return $this->callServer("GET", "/taskLists/$task_list_id");
    }
	
	/**
     * Creates a new task list based on the submitted JSON data.
     *
     * @section TaskLists
	 * @docs http://apidocs.apollohq.com/taskLists
     * @example createTaskList.php
     *
	 * @param integer $project_id the id of the project
	 * @param array $task_list_data an associative array with the task list data
     * @return new task list id
     */
    public function createTaskList($project_id, $task_list_data) {
		$params = $this->adjustParameters($task_list_data, "task-list");
        return $this->callServer("POST", "/projects/$project_id/taskLists", $params);
    }
	
	/**
     * Updates the specified task list record with the changes indicated by the submitted JSON data.
     *
     * @section TaskLists
	 * @docs http://apidocs.apollohq.com/taskLists
     * @example updateTaskList.php
     *
	 * @param integer $task_list_id the task list id
	 * @param array $task_list_data an associative array with the task list data
     */
    public function updateTaskList($task_list_id, $task_list_data) {
		$params = $this->adjustParameters($task_list_data, "task-list");
        return $this->callServer("PUT", "/taskLists/$task_list_id", $params);
    }
	
	/**
     * Destroys the given task list and all of its associated task items. 
     *
     * @section TaskLists
	 * @docs http://apidocs.apollohq.com/taskLists
     * @example deleteTaskList.php
     *
	 * @param integer $task_list_id the task list id
     */
    public function deleteTaskList($task_list_id) {
        return $this->callServer("DELETE", "/taskLists/$task_list_id");
    }
	
	/**
     * Reorders the lists in the project according to the ordering given. Any lists that are not explicitly 
	 * specified will be positioned after the lists that are specified.
     *
     * @section TaskLists
	 * @docs http://apidocs.apollohq.com/taskLists
     * @example reorderTaskLists.php
     *
	 * @param integer $project_id the project id
	 * @param array $taskLists an array with the ids of the task lists that you want to reorder
     */
    public function reorderTaskLists($project_id, $taskLists) {
        $params = $this->adjustParameters($taskLists, "task-lists");
		return $this->callServer("POST", "/projects/$project_id/taskLists/reorder", $params);
    }
	
	/**
     * Returns all the milestones in the given project. The {filter} parameter is optional. Possible values are: 
	 * "all" (the default value), "late", "completed" and "upcoming". 
     *
     * @section Milestones
     * @docs http://apidocs.apollohq.com/milestones
     * @example getMilestones.php
     *
     * @return list of milestones
     */
    public function getMilestones($project_id, $filter = NULL) {
        return $this->callServer("GET", "/projects/$project_id/milestones".($filter !== NULL ? "/$filter" : ""));
    }
	
	/**
     * Returns a single milestone record identified by its integer ID.
     *
     * @section Milestones
     * @docs http://apidocs.apollohq.com/milestones
     * @example getMilestone.php
     *
     * @return milestone data
     */
    public function getMilestone($milestone_id) {
        return $this->callServer("GET", "/milestones/$milestone_id");
    }
	
	/**
     * Creates a new milestone.
     *
     * @section Milestones
	 * @docs http://apidocs.apollohq.com/milestones
     * @example createMilestone.php
     *
	 * @param integer $project_id the id of the project
	 * @param array $milestone an associative array with the milestone data
     * @return new milestone id
     */
    public function createMilestone($project_id, $milestone_data) {
		$params = $this->adjustParameters($milestone_data, "milestone");
        return $this->callServer("POST", "/projects/$project_id/milestones", $params);
    }
	
	/**
     * Updates an existing milestone. You can use this to shift the deadline of a single milestone, and optionally 
	 * shift the deadlines of subsequent milestones as well. Note that you can complete a milestone by sending 
	 * the "completed-at" field.
     *
     * @section Milestones
	 * @docs http://apidocs.apollohq.com/milestones
     * @example updateMilestone.php
     *
	 * @param integer $milestone_id the milestone id
	 * @param array $milestone_data an associative array with the milestone data
     */
    public function updateMilestone($milestone_id, $milestone_data) {
		$params = $this->adjustParameters($milestone_data, "milestone");
        return $this->callServer("PUT", "/milestones/$milestone_id", $params);
    }
	
	/**
     * Destroys the given milestone and all of its associated comments.
     *
     * @section Milestones
	 * @docs http://apidocs.apollohq.com/milestones
     * @example deleteMilestone.php
     *
	 * @param integer $milestone_id the task list id
     */
    public function deleteMilestone($milestone_id) {
        return $this->callServer("DELETE", "/milestones/$milestone_id");
    }
	
	/**
     * Return a list of the 50 most recent comments associated with the specified resource, where the resource 
	 * named in the URL can be one of messages, milestones, tasks, notes or writeboards. The {page} parameter is 
	 * optional. For example, to fetch the most recent comments for the task item with an id of 1, you would use 
	 * the path: /tasks/1/comments.
	 * The root element has a "count" attribute specifying the total number of comments for the resource. If there 
	 * are older comments not included in the response, the root response element will also have a "continued-at" 
	 * attribute specifying the path where the next oldest 50 comments can be retrieved. 
     *
     * @section Comments
     * @docs http://apidocs.apollohq.com/comments
     * @example getComments.php
     *
     * @return list of comments
     */
    public function getComments($resource, $resource_id, $page = 1) {
        return $this->callServer("GET", "/$resource/$resource_id/comments/$page");
    }
	
	/**
     * Retrieve a specific comment by its ID.
     *
     * @section Comments
     * @docs http://apidocs.apollohq.com/comments
     * @example getComment.php
     *
     * @return comment data
     */
    public function getComment($comment_id) {
        return $this->callServer("GET", "/comments/$comment_id");
    }
	
	/**
     * Create a new comment, associating it with a specific resource, where the resource named in the URL can be 
	 * one of messages, milestones, tasks, notes or writeboards. For example, to create a comment for the milestone 
	 * with an ID of 1, you would use the path: /milestones/1/comments. Note that you can also upload files using 
	 * this function, but you need to upload the files first and then attach them.
     *
     * @section Comments
	 * @docs http://apidocs.apollohq.com/comments
     * @example createComment.php
     *
	 * @param integer $resource the type of the resource
	 * @param integer $resource_id the id of the resource
	 * @param array $comment_data an associative array with the comment data
     * @return new comment id
     */
    public function createComment($resource, $resource_id, $comment_data) {
		$params = $this->adjustParameters($comment_data, "comment");
        return $this->callServer("POST", "/$resource/$resource_id/comments", $params);
    }
	
	/**
     * Update a specific comment. Note that you can also upload files using this function, but you need to upload 
	 * the files first and then attach them.
     *
     * @section Comments
	 * @docs http://apidocs.apollohq.com/comments
     * @example updateComment.php
     *
	 * @param integer $comment_id the comment id
	 * @param array $comment_data an associative array with the comment data
     */
    public function updateComment($comment_id, $comment_data) {
		$params = $this->adjustParameters($comment_data, "comment");
        return $this->callServer("PUT", "/comments/$comment_id", $params);
    }
	
	/**
     * Delete the comment with the given ID. 
     *
     * @section Comments
	 * @docs http://apidocs.apollohq.com/comments
     * @example deleteComment.php
     *
	 * @param integer $comment_id the message id
     */
    public function deleteComment($comment_id) {
        return $this->callServer("DELETE", "/comments/$comment_id");
    }
	
	/**
     * Returns all categories for the given project. To filter by type, pass the "type" parameter, which can be one 
	 * of "message" or "file". 
     *
     * @section Categories
     * @docs http://apidocs.apollohq.com/categories
     * @example getCategories.php
     *
     * @return list of categories
     */
    public function getCategories($project_id, $filter = NULL) {
        return $this->callServer("GET", "/projects/$project_id/categories".($filter !== NULL ? "/$filter" : ""));
    }
	
	/**
     * Returns a single category identified by its integer ID.
     *
     * @section Categories
	 * @docs http://apidocs.apollohq.com/categories
     * @example getCategory.php
     *
	 * @param integer $category_id the ID of the category
     * @return category data
     */
    public function getCategory($category_id) {
        return $this->callServer("GET", "/categories/$category_id");
    }
	
	/**
     * Creates a new category of the given type for the given project. The "type" attribute is required and must be 
	 * one of "message" or "file".
     *
     * @section Categories
	 * @docs http://apidocs.apollohq.com/categories
     * @example createCategory.php
     *
	 * @param integer $project_id the id of the project
	 * @param array $category_data an associative array with the category data
     * @return new category id
     */
    public function createCategory($project_id, $category_data) {
		$params = $this->adjustParameters($category_data, "category");
        return $this->callServer("POST", "/projects/$project_id/categories", $params);
    }
	
	/**
     * Updates an existing category identified by its integer ID.
     *
     * @section Categories
	 * @docs http://apidocs.apollohq.com/categories
     * @example updateCategory.php
     *
	 * @param integer $category_id the category id
	 * @param array $category_data an associative array with the category data
     */
    public function updateCategory($category_id, $category_data) {
		$params = $this->adjustParameters($category_data, "category");
        return $this->callServer("PUT", "/categories/$category_id", $params);
    }
	
	/**
     * Deletes the category identified by its integer ID. 
     *
     * @section Categories
	 * @docs http://apidocs.apollohq.com/categories
     * @example deleteCategory.php
     *
	 * @param integer $category_id the category id
     */
    public function deleteCategory($category_id) {
        return $this->callServer("DELETE", "/categories/$category_id");
    }
	
	/**
     * Upload a file. 
     *
     * @section Files
	 * @docs http://apidocs.apollohq.com
     * @example uploadFile.php
     *
	 * @param string $file_path the file path
     */
    public function uploadFile($file_path) {
        $this->errorMessage = "";
		
		if(!file_exists($file_path)) {
			$this->errorMessage = "Unable to read file: $file_path";
            return false;
		}
      
		$file_content = file_get_contents($file_path);
		srand((double)microtime()*1000000); 
		$boundary = "---------------------------".substr(md5(rand(0,32000)),0,10);
        
		$data = "--$boundary\r\n";
        $data .= "Content-Disposition: form-data; name=\"".basename($file_path)."\"; filename=\"$file_path\"\r\n";
        $data .= "Content-Type: application/octet-stream\r\n";
		$data .= "Content-length: " . filesize($file_path) . "\r\n";
        $data .= "Connection: close \r\n\r\n";
        $data .= $file_content;
		$data .= "\r\n";
		$data .= "--$boundary\r\n";
		
        $payload = "POST /". $this->endpoint ."/upload HTTP/1.0\r\n";
        $payload .= "Host: " . $this->apiHost . "\r\n";
        $payload .= "Authorization: Basic " . base64_encode($this->api_key . ":X") . "\r\n";
        $payload .= "User-Agent: ApolloAPI/" . $this->version ."\r\n";
		$payload .= "Content-type: multipart/form-data; boundary=$boundary\r\n";
		$payload .= "Content-length: " . strlen($data) . "\r\n\r\n";
		$payload .= $data;
		
        ob_start();
        $sock = fsockopen("ssl://".$this->apiHost, 443, $errno, $errstr, 30);
        if(!$sock) {
            $this->errorMessage = "Could not connect (ERR $errno: $errstr)";
            ob_end_clean();
            return false;
        }
        
        $response = "";
        fputs($sock, $payload);
        stream_set_timeout($sock, $this->timeout);
        $info = stream_get_meta_data($sock);
        while ((!feof($sock)) && (!$info["timed_out"])) {
            $response .= fread($sock, $this->chunkSize);
            $info = stream_get_meta_data($sock);
        }
        fclose($sock);
        ob_end_clean();
        if ($info["timed_out"]) {
            $this->errorMessage = "Could not read response (timed out)";
            return false;
        }

        list($headers, $response) = explode("\r\n\r\n", $response, 2);
        $headers = explode("\r\n", $headers);
		$this->errored = false;
		foreach($headers as $h){
            if(preg_match("/^HTTP\/1\.0 /", $h, $matches) > 0) {
                $this->responseStatus = substr($h, 9, 3);
                break;
            }
        }
		if(substr($this->responseStatus, 0, 1) != 2) {
			$this->errored = true;
		}
        
        $response = stripslashes($response);
        
        if($this->errored) {
            $this->errorMessage = $response;
            return false;
        }
        
        return $response;
    }
	
	/**
     * Download a file identified by its integer ID.
     *
     * @section Files
	 * @docs http://apidocs.apollohq.com/files
     * @example downloadFile.php
     *
	 * @param integer $file_id the ID of the file
     * @return file data
     */
    public function downloadFile($file_id) {
        return $this->callServer("GET", "/download/$file_id");
    }
	
	/**
     * Returns a list of the 50 most recent files (attachments) in the given project that are visible to the 
	 * authenticated user.The {page} parameter is optional. If there are older files not included in the response, 
	 * the root response element will also have a "continued-at" attribute specifying the path where the next 
	 * oldest 50 files can be retrieved.
     *
     * @section Files
     * @docs http://apidocs.apollohq.com/files
     * @example getFiles.php
     *
     * @return list of files
     */
    public function getFiles($project_id, $page = 1) {
        return $this->callServer("GET", "/projects/$project_id/files/$page");
    }
	
	/**
     * Returns a list of the 50 most recent files (attachments) in the given project that are visible to the 
	 * authenticated user and that belongs to the specified category. The {page} parameter is optional. If there 
	 * are older files not included in the response, the root response element will also have a "continued-at" 
	 * attribute specifying the path where the next oldest 50 files can be retrieved.
     *
     * @section Files
     * @docs http://apidocs.apollohq.com/files
     * @example getFilesByCategory.php
     *
     * @return list of files
     */
    public function getFilesByCategory($project_id, $category_id, $page = 1) {
        return $this->callServer("GET", "/projects/$project_id/cat/$category_id/files/$page");
    }
	
	/**
     * Returns a single file identified by its integer ID.
     *
     * @section Files
	 * @docs http://apidocs.apollohq.com/files
     * @example getFile.php
     *
	 * @param integer $file_id the ID of the file
     * @return file data
     */
    public function getFile($file_id) {
        return $this->callServer("GET", "/files/$file_id");
    }
	
	/**
     * Returns the currently logged in person (you).
     *
     * @section People
	 * @docs http://apidocs.apollohq.com/people
     * @example getMe.php
     *
     * @return user data
     */
    public function getMe() {
        return $this->callServer("GET", "/me");
    }
	
	/**
     * Returns all people visible to (and including) the requesting user.
     *
     * @section People
	 * @docs http://apidocs.apollohq.com/people
     * @example getPeople.php
     *
     * @return user data
     */
    public function getPeople() {
        return $this->callServer("GET", "/people");
    }
	
	/**
     * Returns all people with access to the given project.
     *
     * @section People
	 * @docs http://apidocs.apollohq.com/people
     * @example getProjectPeople.php
     *
	 * @param integer $project_id the ID of the project
     * @return user data
     */
    public function getProjectPeople($project_id) {
        return $this->callServer("GET", "/projects/$project_id/people");
    }
	
	/**
     * Returns all people from the given company that are visible to the requesting user.
     *
     * @section People
	 * @docs http://apidocs.apollohq.com/people
     * @example getCompanyPeople.php
     *
	 * @param integer $company_id the ID of the company
     * @return user data
     */
    public function getCompanyPeople($company_id) {
        return $this->callServer("GET", "/companies/$company_id/people");
    }
	
	/**
     * Returns a single person identified by their integer ID.
     *
     * @section People
	 * @docs http://apidocs.apollohq.com/people
     * @example getPerson.php
     *
	 * @param integer $person_id the ID of the person
     * @return user data
     */
    public function getPerson($person_id) {
        return $this->callServer("GET", "/people/$person_id");
    }
	
	/**
     * Returns a list of all companies visible to the requesting user.
     *
     * @section Companies
	 * @docs http://apidocs.apollohq.com/companies
     * @example getCompanies.php
     *
     * @return companies data
     */
    public function getCompanies() {
        return $this->callServer("GET", "/companies");
    }
	
	/**
     * Returns a list of all companies associated with the given project.
     *
     * @section Companies
	 * @docs http://apidocs.apollohq.com/companies
     * @example getProjectCompanies.php
     *
	 * @param integer $project_id the ID of the project
     * @return companies data
     */
    public function getProjectCompanies($project_id) {
        return $this->callServer("GET", "/projects/$project_id/companies");
    }
	
	/**
     * Returns a single company identified by its integer ID.
     *
     * @section Companies
	 * @docs http://apidocs.apollohq.com/companies
     * @example getCompany.php
     *
	 * @param integer $company_id the ID of the company
     * @return company data
     */
    public function getCompany($company_id) {
        return $this->callServer("GET", "/companies/$company_id");
    }
	
	/**
     * Returns a page full of time entries for the given project, in descending order by date. Each page contains 
	 * up to 50 time entry records. To select a different page of data, set the "page" query parameter to a value 
	 * greater than one.
     *
     * @section TimeEntries
     * @docs http://apidocs.apollohq.com/timeEntries
     * @example getProjectTime.php
     *
     * @return list of time entries
     */
    public function getProjectTime($project_id, $page = 1) {
        return $this->callServer("GET", "/projects/$project_id/timeEntries/$page");
    }
	
	/**
     * Returns all time entries associated with the given task item, in descending order by date.
     *
     * @section TimeEntries
     * @docs http://apidocs.apollohq.com/timeEntries
     * @example getTaskTime.php
     *
     * @return list of time entries
     */
    public function getTaskTime($task_id) {
        return $this->callServer("GET", "/tasks/$task_id/timeEntries");
    }
	
	/**
     * Returns a page full of time entries for the given contact, in descending order by date. Each page contains 
	 * up to 50 time entry records. To select a different page of data, set the "page" query parameter to a value 
	 * greater than one.
     *
     * @section TimeEntries
     * @docs http://apidocs.apollohq.com/timeEntries
     * @example getContactTime.php
     *
     * @return list of time entries
     */
    public function getContactTime($contact_id, $page = 1) {
        return $this->callServer("GET", "/contacts/$contact_id/timeEntries/$page");
    }
	
	/**
     * Returns a single time entry record identified by its integer ID.
     *
     * @section TimeEntries
	 * @docs http://apidocs.apollohq.com/timeEntries
     * @example getTimeEntry.php
     *
	 * @param integer $time_entry_id the ID of the time entry
     * @return time entry data
     */
    public function getTimeEntry($time_entry_id) {
        return $this->callServer("GET", "/timeEntries/$time_entry_id");
    }
	
	/**
     * Creates a new time entry for the given project.
     *
     * @section TimeEntries
	 * @docs http://apidocs.apollohq.com/timeEntries
     * @example createProjectTime.php
     *
	 * @param integer $project_id the id of the project
	 * @param array $time_entry_data an associative array with the time entry data
     * @return new time entry id
     */
    public function createProjectTime($project_id, $time_entry_data) {
		$params = $this->adjustParameters($time_entry_data, "time-entry");
        return $this->callServer("POST", "/projects/$project_id/timeEntries", $params);
    }
	
	/**
     * Creates a new time entry for the given task.
     *
     * @section TimeEntries
	 * @docs http://apidocs.apollohq.com/timeEntries
     * @example createTaskTime.php
     *
	 * @param integer $task_id the id of the task
	 * @param array $time_entry_data an associative array with the time entry data
     * @return new time entry id
     */
    public function createTaskTime($task_id, $time_entry_data) {
		$params = $this->adjustParameters($time_entry_data, "time-entry");
        return $this->callServer("POST", "/tasks/$task_id/timeEntries", $params);
    }
	
	/**
     * Creates a new time entry for the given contact.
     *
     * @section TimeEntries
	 * @docs http://apidocs.apollohq.com/timeEntries
     * @example createContactTime.php
     *
	 * @param integer $contact_id the id of the contact
	 * @param array $time_entry_data an associative array with the time entry data
     * @return new time entry id
     */
    public function createContactTime($contact_id, $time_entry_data) {
		$params = $this->adjustParameters($time_entry_data, "time-entry");
        return $this->callServer("POST", "/contacts/$contact_id/timeEntries", $params);
    }
	
	/**
     * Updates the given time-entry record with the data given.
     *
     * @section TimeEntries
	 * @docs http://apidocs.apollohq.com/timeEntries
     * @example updateTime.php
     *
	 * @param integer $time_entry_id the time entry id
	 * @param array $time_entry_data an associative array with the time entry data
     */
    public function updateTime($time_entry_id, $time_entry_data) {
		$params = $this->adjustParameters($time_entry_data, "time-entry");
        return $this->callServer("PUT", "/timeEntries/$time_entry_id", $params);
    }
	
	/**
     * Destroys the given time entry record.
     *
     * @section TimeEntries
	 * @docs http://apidocs.apollohq.com/timeEntries
     * @example deleteTime.php
     *
	 * @param integer $time_entry_id the time entry id
     */
    public function deleteTime($time_entry_id) {
        return $this->callServer("DELETE", "/timeEntries/$time_entry_id");
    }
	
	/**
     * Return the set of time entries that match the given criteria. This action accepts the following query 
	 * parameters: "from", "to", "user_id", "task_id", "project_id" and "company_id". Both "from" and "to" should 
	 * be dates in "YYYYMMDD" format (or in any other format accepted by the PHP function "date()"), and can be 
	 * used to restrict the result to a particular date range (No more than 6 months' worth of entries may be 
	 * returned in a single query, though). The "user_id" parameter lets you constrain the result to a single 
	 * person's time entries. "task_id" restricts the result to only those entries relating to the given task 
	 * item. "project_id" restricts the entries to those for the given project, and "company_id" restricts the 
	 * entries to those for the given company.
     *
     * @section TimeEntries
     * @docs http://apidocs.apollohq.com/timeEntries
     * @example getTimeReport.php
     *
	 * @param array $params an associative array with the query parameters
     * @return list of time entries
     */
    public function getTimeReport($params = array()) {
        return $this->callServer("GET", "/report/timeEntries", $params);
    }
	
	/**
     * Returns info about the current Apollo account, its subscription, the default post and attachment categories, 
	 * the time entry categories and the project categories.
     *
     * @section Account
     * @docs http://apidocs.apollohq.com/account
     * @example getAccount.php
     *
     * @return account data
     */
    public function getAccount() {
        return $this->callServer("GET", "/account");
    }
	
	/**
     * Returns a collection of contacts that are visible to the authenticated user. Use the search query to refine 
	 * your search. Available parameters are:
	 * 		- "page": if the results are many you can jump to the next page using the "page" parameter.
     *		- "contact-type": possible values are person or company (if you don't pass nothing both person and 
	 *		  company are returned).
     * 		- "tags": search by tags (multiple tags are comma-separated). Note that if you search by tags you 
	 * 		  cannot use the following parameters.
     * 		- "title": search by title. Note that you cannot use this parameter if you search by tags.
     * 		- "company-id": search by company-id. Note that you cannot use this parameter if you search by tags.
     * 		- "term": search a term contained in a person or a company name. Note that you cannot use this 
	 * 		  parameter if you search by tags.
     *
     * @section Contacts
     * @docs http://apidocs.apollohq.com/contacts
     * @example getContacts.php
     *
	 * @param array $params an associative array with the query parameters
     * @return list of contacts
     */
    public function getContacts($params = array()) {
        return $this->callServer("GET", "/contacts", $params);
    }
	
	/**
     * Returns a collection of contacts that are visible to the authenticated user. "since" should be a date in 
	 * "YYYYMMDDHHIISS" format (or in any other format accepted by the PHP function "date()"). 
     *
     * @section Contacts
     * @docs http://apidocs.apollohq.com/contacts
     * @example getContactsSince.php
     *
	 * @param string $since a date
	 * @param integer $page
     * @return list of contacts
     */
    public function getContactsSince($since, $page = 1) {
        return $this->callServer("GET", "/contacts?since=$since&page=$page");
    }
	
	/**
     * Returns a collection of recently viewed contacts. 
     *
     * @section Contacts
     * @docs http://apidocs.apollohq.com/contacts
     * @example getRecentlyViewedContacts.php
     *
	 * @param integer $page
     * @return list of contacts
     */
    public function getRecentlyViewedContacts($page = 1) {
        return $this->callServer("GET", "/contacts/recentlyViewed/$page");
    }
	
	/**
     * Returns a single contact. 
     *
     * @section Contacts
	 * @docs http://apidocs.apollohq.com/contacts
     * @example getContact.php
     *
	 * @param integer $contact_id the ID of the contact
     * @return contact data
     */
    public function getContact($contact_id) {
        return $this->callServer("GET", "/contacts/$contact_id");
    }
	
	/**
     * Creates a new person or company with the currently authenticated user as the author. Additionally (in the 
	 * case you are creating a person), the company-name is used to either lookup a company with that name or 
	 * create a new one if it didn't already exist. You can also refer to an existing company instead using 
	 * company-id. By default, a new contact is assumed to be visible to everyone. You can also chose to make the 
	 * contact only visible to the creator using "only-me" as the value for the visibility key. Or "group" and 
	 * pass in a group-id key too. Or "selected-people" and pass an array of id of users that can see the contact 
	 * (note that the creator id is always included). If the account doesn't allow for more contacts to be created, 
	 * a "507 Insufficient Storage" response will be returned.
     *
     * @section Contacts
	 * @docs http://apidocs.apollohq.com/contacts
     * @example createContact.php
     *
	 * @param array $contact_data an associative array with the contact data
     * @return new contact id
     */
    public function createContact($contact_data) {
		$params = $this->adjustParameters($contact_data, "contact");
        return $this->callServer("POST", "/contacts", $params);
    }
	
	/**
     * Updates an existing contact with new details from the submitted JSON. Contact data, important dates and 
	 * addresses data that include an id will be updated, data that doesn't will be assumed to be new and created 
	 * from scratch. To remove a piece of data, prefix its id with a minus sign (e.g. "-1").
     *
     * @section Contacts
	 * @docs http://apidocs.apollohq.com/contacts
     * @example updateContact.php
     *
	 * @param integer $contact_id the contact id
	 * @param array $contact_data an associative array with the contact data
     */
    public function updateContact($contact_id, $contact_data) {
		$params = $this->adjustParameters($contact_data, "contact");
        return $this->callServer("PUT", "/contacts/$contact_id", $params);
    }
	
	/**
     * Destroys the given contact. 
     *
     * @section Contacts
	 * @docs http://apidocs.apollohq.com/contacts
     * @example deleteContact.php
     *
	 * @param integer $contact_id the contact id
     */
    public function deleteContact($contact_id) {
        return $this->callServer("DELETE", "/contacts/$contact_id");
    }
	
	/**
     * Returns a collection of cases that are visible to the authenticated user. Possible values for "status" 
	 * parameter can be open or closed (if you don't pass nothing both open and closed cases are returned).  
     *
     * @section Cases
     * @docs http://apidocs.apollohq.com/cases
     * @example getCases.php
     *
	 * @param string $status
	 * @param integer $page
     * @return list of cases
     */
    public function getCases($status = "", $page = 1) {
        return $this->callServer("GET", "/cases?status=$status&page=$page");
    }
	
	/**
     * Returns a single case. 
     *
     * @section Cases
	 * @docs http://apidocs.apollohq.com/cases
     * @example getCase.php
     *
	 * @param integer $case_id the ID of the case
     * @return case data
     */
    public function getCase($case_id) {
        return $this->callServer("GET", "/cases/$case_id");
    }
	
	/**
     * Creates a new case with the currently authenticated user as the author. By default, a new case is assumed to 
	 * be visible to everyone. You can also chose to make the case only visible to the creator using "only-me" as 
	 * the value for the visibility key. Or "group" and pass in a group-id key too. Or "selected-people" and pass 
	 * an array of id of users that can see the case (note that the creator id is always included). If the account 
	 * doesn't allow for more cases to be created, a "507 Insufficient Storage" response will be returned.
     *
     * @section Cases
	 * @docs http://apidocs.apollohq.com/cases
     * @example createCase.php
     *
	 * @param array $case_data an associative array with the case data
     * @return new case id
     */
    public function createCase($case_data) {
		$params = $this->adjustParameters($case_data, "case");
        return $this->callServer("POST", "/cases", $params);
    }
	
	/**
     * Updates an existing case with new details from the submitted JSON.
     *
     * @section Cases
	 * @docs http://apidocs.apollohq.com/cases
     * @example updateCase.php
     *
	 * @param integer $case_id the case id
	 * @param array $case_data an associative array with the case data
     */
    public function updateCase($case_id, $case_data) {
		$params = $this->adjustParameters($case_data, "case");
        return $this->callServer("PUT", "/cases/$case_id", $params);
    }
	
	/**
     * Destroys the given case. 
     *
     * @section Cases
	 * @docs http://apidocs.apollohq.com/cases
     * @example deleteCase.php
     *
	 * @param integer $case_id the case id
     */
    public function deleteCase($case_id) {
        return $this->callServer("DELETE", "/cases/$case_id");
    }
	
	/**
     * Returns a collection of deals that are visible to the authenticated user. Possible values for "status" 
	 * parameter can be pending, win or lost (if you don't pass nothing all deals are returned). The "currency" 
	 * field is a three-character ISO 4217 currency identifier. The "price-type" field indicates whether the deal's 
	 * bid has a fixed price ("fixed"), or is based on an hourly ("hour"), monthly ("month") or annual ("year") 
	 * price. The "duration" field indicates how many "price-type" units long the deal is expected to span. The 
	 * "deal-status" field indicates whether the deal is "pending", "won", or "lost". The "deal-responsible-id" 
	 * field identifies which user is responsible for this deal. The "deal-category-id" field is the id of the 
	 * category record for this deal. The "background-info" field is arbitrary text describing this deal. The 
	 * "involved-people" field is a list of the people and companies involved with this deal. The "deal-with" 
	 * field is the person or company the deal is with. 
     *
     * @section Deals
     * @docs http://apidocs.apollohq.com/deals
     * @example getDeals.php
     *
	 * @param string $status
	 * @param integer $page
     * @return list of deals
     */
    public function getDeals($status = "", $page = 1) {
        return $this->callServer("GET", "/deals?status=$status&page=$page");
    }
	
	/**
     * Returns a single deal. 
     *
     * @section Deals
	 * @docs http://apidocs.apollohq.com/deals
     * @example getDeal.php
     *
	 * @param integer $deal_id the ID of the deal
     * @return deal data
     */
    public function getDeal($deal_id) {
        return $this->callServer("GET", "/deals/$deal_id");
    }
	
	/**
     * Creates a new deal with the currently authenticated user as the author. By default, a new deal is assumed to 
	 * be visible to everyone. You can also chose to make the deal only visible to the creator using "only-me" as 
	 * the value for the visibility key. Or "group" and pass in a group-id key too. Or "selected-people" and pass 
	 * an array of id of users that can see the deal (note that the creator id is always included). If the account 
	 * doesn't allow for more deals to be created, a "507 Insufficient Storage" response will be returned.
     *
     * @section Deals
	 * @docs http://apidocs.apollohq.com/deals
     * @example createDeal.php
     *
	 * @param array $deal_data an associative array with the deal data
     * @return new deal id
     */
    public function createDeal($deal_data) {
		$params = $this->adjustParameters($deal_data, "deal");
        return $this->callServer("POST", "/deals", $params);
    }
	
	/**
     * Updates an existing deal with new details from the submitted JSON.
     *
     * @section Deals
	 * @docs http://apidocs.apollohq.com/deals
     * @example updateDeal.php
     *
	 * @param integer $deal_id the deal id
	 * @param array $deal_data an associative array with the deal data
     */
    public function updateDeal($deal_id, $deal_data) {
		$params = $this->adjustParameters($deal_data, "deal");
        return $this->callServer("PUT", "/deals/$deal_id", $params);
    }
	
	/**
     * Destroys the given deal. 
     *
     * @section Deals
	 * @docs http://apidocs.apollohq.com/deals
     * @example deleteDeal.php
     *
	 * @param integer $deal_id the deal id
     */
    public function deleteDeal($deal_id) {
        return $this->callServer("DELETE", "/deals/$deal_id");
    }
	
	/**
     * Returns all categories.
     *
     * @section DealCategories
     * @docs http://apidocs.apollohq.com/dealCategories
     * @example getDealCategories.php
     *
     * @return list of deal categories
     */
    public function getDealCategories() {
        return $this->callServer("GET", "/dealCategories");
    }
	
	/**
     * Returns a single category identified by its integer ID.
     *
     * @section DealCategories
	 * @docs http://apidocs.apollohq.com/dealCategories
     * @example getDealCategory.php
     *
	 * @param integer $category_id the ID of the category
     * @return deal category data
     */
    public function getDealCategory($category_id) {
        return $this->callServer("GET", "/dealCategories/$category_id");
    }
	
	/**
     * Creates a new category.
     *
     * @section DealCategories
	 * @docs http://apidocs.apollohq.com/dealCategories
     * @example createDealCategory.php
     *
	 * @param array $category_data an associative array with the category data
     * @return new category id
     */
    public function createDealCategory($category_data) {
		$params = $this->adjustParameters($category_data, "category");
        return $this->callServer("POST", "/dealCategories", $params);
    }
	
	/**
     * Updates an existing category identified by its integer ID.
     *
     * @section DealCategories
	 * @docs http://apidocs.apollohq.com/dealCategories
     * @example updateDealCategory.php
     *
	 * @param integer $category_id the category id
	 * @param array $category_data an associative array with the category data
     */
    public function updateDealCategory($category_id, $category_data) {
		$params = $this->adjustParameters($category_data, "category");
        return $this->callServer("PUT", "/dealCategories/$category_id", $params);
    }
	
	/**
     * Deletes the category identified by its integer ID.
     *
     * @section DealCategories
	 * @docs http://apidocs.apollohq.com/dealCategories
     * @example deleteDealCategories.php
     *
	 * @param integer $category_id the category id
     */
    public function deleteDealCategory($category_id) {
        return $this->callServer("DELETE", "/dealCategories/$category_id");
    }
	
	/**
     * Returns a collection of notes that are visible to the authenticated user and related to a specific contact. 
     *
     * @section Notes
     * @docs http://apidocs.apollohq.com/notes
     * @example getContactNotes.php
     *
	 * @param integer $contact_id the contact id
     * @return list of notes
     */
    public function getContactNotes($contact_id, $page = 1) {
        return $this->callServer("GET", "/contacts/$contact_id/notes/$page");
    }
	
	/**
     * Returns a collection of notes that are visible to the authenticated user and related to a specific case. 
     *
     * @section Notes
     * @docs http://apidocs.apollohq.com/notes
     * @example getCaseNotes.php
     *
	 * @param integer $case_id the case id
     * @return list of notes
     */
    public function getCaseNotes($case_id, $page = 1) {
        return $this->callServer("GET", "/cases/$case_id/notes/$page");
    }
	
	/**
     * Returns a collection of notes that are visible to the authenticated user and related to a specific deal. 
     *
     * @section Notes
     * @docs http://apidocs.apollohq.com/notes
     * @example getDealNotes.php
     *
	 * @param integer $deal_id the deal id
     * @return list of notes
     */
    public function getDealNotes($deal_id, $page = 1) {
        return $this->callServer("GET", "/deals/$deal_id/notes/$page");
    }
	
	/**
     * Returns a single note. Attachments are included, but comments are kept separate at /notes/#{id}/comments. 
     *
     * @section Notes
	 * @docs http://apidocs.apollohq.com/notes
     * @example getNote.php
     *
	 * @param integer $note_id the ID of the note
     * @return note data
     */
    public function getNote($note_id) {
        return $this->callServer("GET", "/notes/$note_id");
    }
	
	/**
     * Creates a new note with the currently authenticated user as the author. By default, a new note is assumed 
	 * to be visible to everyone. You can also choose to make the note only visible to the creator using "only-me" 
	 * as the value for the visibility key. Or "group" and pass in a group-id key too. Or "selected-people" and 
	 * pass an array of id of users that can see the note (note that the creator id is always included).
     *
     * @section Notes
	 * @docs http://apidocs.apollohq.com/notes
     * @example createContactNote.php
     *
	 * @param integer $contact_id the id of the contact
	 * @param array $note_data an associative array with the note data
     * @return new note id
     */
    public function createContactNote($contact_id, $note_data) {
		$params = $this->adjustParameters($note_data, "note");
        return $this->callServer("POST", "/contacts/$contact_id/notes", $params);
    }
	
	/**
     * Creates a new note with the currently authenticated user as the author. By default, a new note is assumed 
	 * to be visible to everyone. You can also choose to make the note only visible to the creator using "only-me" 
	 * as the value for the visibility key. Or "group" and pass in a group-id key too. Or "selected-people" and 
	 * pass an array of id of users that can see the note (note that the creator id is always included).
     *
     * @section Notes
	 * @docs http://apidocs.apollohq.com/notes
     * @example createCaseNote.php
     *
	 * @param integer $case_id the id of the case
	 * @param array $note_data an associative array with the note data
     * @return new note id
     */
    public function createCaseNote($case_id, $note_data) {
		$params = $this->adjustParameters($note_data, "note");
        return $this->callServer("POST", "/cases/$case_id/notes", $params);
    }
	
	/**
     * Creates a new note with the currently authenticated user as the author. By default, a new note is assumed 
	 * to be visible to everyone. You can also choose to make the note only visible to the creator using "only-me" 
	 * as the value for the visibility key. Or "group" and pass in a group-id key too. Or "selected-people" and 
	 * pass an array of id of users that can see the note (note that the creator id is always included).
     *
     * @section Notes
	 * @docs http://apidocs.apollohq.com/notes
     * @example createDealNote.php
     *
	 * @param integer $deal_id the id of the deal
	 * @param array $note_data an associative array with the note data
     * @return new note id
     */
    public function createDealNote($deal_id, $note_data) {
		$params = $this->adjustParameters($note_data, "note");
        return $this->callServer("POST", "/deals/$deal_id/notes", $params);
    }
	
	/**
     * Updates an existing note.
     *
     * @section Notes
	 * @docs http://apidocs.apollohq.com/notes
     * @example updateNote.php
     *
	 * @param integer $note_id the note id
	 * @param array $note_data an associative array with the note data
     */
    public function updateNote($note_id, $note_data) {
		$params = $this->adjustParameters($note_data, "note");
        return $this->callServer("PUT", "/notes/$note_id", $params);
    }
	
	/**
     * Destroys the given note.
     *
     * @section Notes
	 * @docs http://apidocs.apollohq.com/notes
     * @example deleteNote.php
     *
	 * @param integer $note_id the note id
     */
    public function deleteNote($note_id) {
        return $this->callServer("DELETE", "/notes/$note_id");
    }
	
	/**
     * Returns all tags used in the workspace. Each tag is associated with the number of contacts tagged with it.
     *
     * @section Tags
     * @docs http://apidocs.apollohq.com/tags
     * @example getTags.php
     *
     * @return list of tags
     */
    public function getTags() {
        return $this->callServer("GET", "/tags");
    }
	
	/**
     * Returns all tags used for the contact.
     *
     * @section Tags
     * @docs http://apidocs.apollohq.com/tags
     * @example getContactTags.php
     *
	 * @param integer $contact_id the contact id
     * @return list of tags
     */
    public function getContactTags($contact_id) {
        return $this->callServer("GET", "/contacts/$contact_id/tags");
    }
	
	/**
     * Adds a tag to a person or a company.
     *
     * @section Tags
	 * @docs http://apidocs.apollohq.com/tags
     * @example createTag.php
     *
	 * @param integer $contact_id the id of the contact
	 * @param string $tags
     */
    public function createTag($contact_id, $tags) {
		$params = $this->adjustParameters($tags, "tags");
        return $this->callServer("POST", "/contacts/$contact_id/tags", $params);
    }
	
	/**
     * Removes a tag from a person or a company.
     *
     * @section Tags
	 * @docs http://apidocs.apollohq.com/tags
     * @example deleteTag.php
     *
	 * @param integer $contact_id the contact id
	 * @param string $tag
     */
    public function deleteTag($contact_id, $tag) {
        return $this->callServer("DELETE", "/contacts/$contact_id/tags/$tag");
    }
	
	/**
     * Returns a collection of groups.
     *
     * @section Workgroups
     * @docs http://apidocs.apollohq.com/workgroups
     * @example getWorkgroups.php
     *
     * @return list of groups
     */
    public function getWorkgroups() {
        return $this->callServer("GET", "/workgroups");
    }
	
	/**
     * Returns a single group and the users associated with it.
     *
     * @section Workgroup
	 * @docs http://apidocs.apollohq.com/workgroups
     * @example getWorkgroup.php
     *
	 * @param integer $workgroup_id the ID of the workgroup
     * @return workgroup data
     */
    public function getWorkgroup($workgroup_id) {
        return $this->callServer("GET", "/workgroups/$workgroup_id");
    }
	
	/**
     * Creates a new group.
     *
     * @section Workgroups
	 * @docs http://apidocs.apollohq.com/workgroups
     * @example createWorkgroup.php
     *
	 * @param array $workgroup_data an associative array with the workgroup data
     * @return new workgroup id
     */
    public function createWorkgroup($workgroup_data) {
		$params = $this->adjustParameters($workgroup_data, "group");
        return $this->callServer("POST", "/workgroups", $params);
    }
	
	/**
     * Updates an existing workgroups identified by its integer ID. To remove a user from the group prefix his 
	 * ID with a -.
     *
     * @section Workgroups
	 * @docs http://apidocs.apollohq.com/workgroups
     * @example updateWorkgroup.php
     *
	 * @param integer $workgroup_id the workgroup id
	 * @param array $workgroup_data an associative array with the workgroup data
     */
    public function updateWorkgroup($workgroup_id, $workgroup_data) {
		$params = $this->adjustParameters($workgroup_data, "group");
        return $this->callServer("PUT", "/workgroups/$workgroup_id", $params);
    }
	
	/**
     * Deletes the group identified by its integer ID. Note that you cannot delete workgroups associated with a 
	 * contact or a note.
     *
     * @section Workgroups
	 * @docs http://apidocs.apollohq.com/workgroups
     * @example deleteWorkgroup.php
     *
	 * @param integer $workgroup_id the workgroup id
     */
    public function deleteWorkgroup($workgroup_id) {
        return $this->callServer("DELETE", "/workgroups/$workgroup_id");
    }
	
	/**
     * Returns recordings (notes, comments, emails). The collection is ordered from oldest to newest. 
     *
     * @section Recordings
	 * @docs http://apidocs.apollohq.com/recordings
     * @example getRecordings.php
     *
	 * @param integer $page
     * @return recordings data
     */
    public function getRecordings($page = 1) {
        return $this->callServer("GET", "/recordings/$page");
    }
	
	/**
     * Returns the recordings (notes, comments, emails) that have updated since the time in the query parameter. 
	 * The collection is ordered from oldest to newest. The since parameter is expressed as yyyymmddhhmmss in the 
	 * UTC timezone. 
     *
     * @section Recordings
	 * @docs http://apidocs.apollohq.com/recordings
     * @example getRecordingsSince.php
     *
	 * @param string $since (UTC timestamp)
	 * @param integer $page
     * @return recordings data
     */
    public function getRecordingsSince($since, $page = 1) {
        return $this->callServer("GET", "/recordings/since/$since/$page");
    }
	
	/**
     * Returns a list of all calendars you can access. 
     *
     * @section Events (LEGACY)
	 * @docs http://apidocs.apollohq.com/legacy/calendars
     * @example getCalendars.php
     *
     * @return calendars data
     */
    public function getCalendars() {
        return $this->callServer("GET", "/calendars");
    }
	
	/**
     * Returns a collection of events that are visible to the authenticated user. Use the search query to refine 
	 * your search. Available parameters are: 
	 * 		- "page": if the results are many you can jump to the next page using the "page" parameter.
     *		- "from": you can ignore events before a certain date.
     *		- "to": you can ignore events after a certain date.
     *		- "calendar-ids": a list of calendar ids in the form of a string (Ex.: '1,3,8').
     *
     * @section Events (LEGACY)
     * @docs http://apidocs.apollohq.com/legacy/events
     * @example getEvents.php
     *
	 * @param array $params an associative array with the query parameters
     * @return list of events
     */
    public function getEvents($params = array()) {
        return $this->callServer("GET", "/calendars/events", $params);
    }
	
	/**
     * Returns a collection of recurrent events that are visible to the authenticated user. Use the search query 
	 * to refine your search. Available parameters are:  
	 * 		- "page": if the results are many you can jump to the next page using the "page" parameter.
     *		- "from": you can ignore events before a certain date.
     *		- "to": you can ignore events after a certain date.
     *		- "calendar-ids": a list of calendar ids in the form of a string (Ex.: '1,3,8').
     *
     * @section Events (LEGACY)
     * @docs http://apidocs.apollohq.com/legacy/events
     * @example getRecurrentEvents.php
     *
	 * @param array $params an associative array with the query parameters
     * @return list of events
     */
    public function getRecurrentEvents($params = array()) {
        return $this->callServer("GET", "/calendars/events/recurrent", $params);
    }
	
	/**
     * Returns a single event record identified by its integer ID.
     *
     * @section Events (LEGACY)
	 * @docs http://apidocs.apollohq.com/legacy/events
     * @example getEvent.php
     *
	 * @param integer $event_id the ID of the event
     * @return event data
     */
    public function getEvent($event_id) {
        return $this->callServer("GET", "/events/$event_id");
    }
	
	/**
     * Creates a new event.
     *
     * @section Events (LEGACY)
	 * @docs http://apidocs.apollohq.com/legacy/events
     * @example createEvent.php
     *
	 * @param array $event_data an associative array with the event data
	 * @param integer $calendar_id the calendar id
     * @return new event id
     */
    public function createEvent($calendar_id, $event_data) {
		$params = $this->adjustParameters($event_data, "event");
        return $this->callServer("POST", "/calendars/$calendar_id/events", $params);
    }
	
	/**
     * Updates an existing event.
     *
     * @section Events (LEGACY)
	 * @docs http://apidocs.apollohq.com/legacy/events
     * @example updateEvent.php
     *
	 * @param integer $event_id the event id
	 * @param array $event_data an associative array with the event data
     */
    public function updateEvent($event_id, $event_data) {
		$params = $this->adjustParameters($event_data, "event");
        return $this->callServer("PUT", "/events/$event_id", $params);
    }
	
	/**
     * Destroys the given event.
     *
     * @section Events (LEGACY)
	 * @docs http://apidocs.apollohq.com/legacy/events
     * @example deleteEvent.php
     *
	 * @param integer $event_id the event id
     */
    public function deleteEvent($event_id) {
        return $this->callServer("DELETE", "/events/$event_id");
    }

    /**
     * Returns a list of all calendars used in the workspace. 
     *
     * @section Events
     * @docs http://apidocs.apollohq.com/calendars
     * @example getCalendars.php
     *
     * @return calendars data
     */
    public function getCalCalendars() {
        return $this->callServer("GET", "/cal-events/calendars");
    }

    /**
     * Returns a list of all event categories used in the workspace. 
     *
     * @section Events
     * @docs http://apidocs.apollohq.com/eventCategories
     * @example getEventCategories.php
     *
     * @return categories data
     */
    public function getCalEventCategories() {
        return $this->callServer("GET", "/cal-events/categories");
    }
    
    /**
     * Returns a collection of events that are visible to the authenticated user. Use the search query to refine 
     * your search. Available parameters are: 
     *      - "page": if the results are many you can jump to the next page using the "page" parameter.
     *      - "from": you can ignore events before a certain date.
     *      - "to": you can ignore events after a certain date.
     *      - "calendar-ids": a list of calendar ids in the form of a string (Ex.: '1,3,8').
     *      - "category-ids": a list of category ids in the form of a string (Ex.: '1,3,8').
     *
     * @section Events
     * @docs http://apidocs.apollohq.com/events
     * @example getEvents.php
     *
     * @param array $params an associative array with the query parameters
     * @return list of events
     */
    public function getCalEvents($params = array()) {
        return $this->callServer("GET", "/cal-events", $params);
    }
    
    /**
     * Returns a single event record identified by its integer ID.
     *
     * @section Events
     * @docs http://apidocs.apollohq.com/events
     * @example getEvent.php
     *
     * @param integer $event_id the ID of the event
     * @return event data
     */
    public function getCalEvent($event_id) {
        return $this->callServer("GET", "/cal-events/$event_id");
    }
    
    /**
     * Creates a new event.
     *
     * @section Events
     * @docs http://apidocs.apollohq.com/events
     * @example createEvent.php
     *
     * @param array $event_data an associative array with the event data
     * @return new event id
     */
    public function createCalEvent($event_data) {
        $params = $this->adjustParameters($event_data, "event");
        return $this->callServer("POST", "/cal-events", $params);
    }
    
    /**
     * Updates an existing event.
     *
     * @section Events
     * @docs http://apidocs.apollohq.com/events
     * @example updateEvent.php
     *
     * @param integer $event_id the event id
     * @param array $event_data an associative array with the event data
     */
    public function updateCalEvent($event_id, $event_data) {
        $params = $this->adjustParameters($event_data, "event");
        return $this->callServer("PUT", "/cal-events/$event_id", $params);
    }
    
    /**
     * Destroys the given event.
     *
     * @section Events
     * @docs http://apidocs.apollohq.com/events
     * @example deleteEvent.php
     *
     * @param integer $event_id the event id
     */
    public function deleteCalEvent($event_id) {
        return $this->callServer("DELETE", "/cal-events/$event_id");
    }

    /**
     * Destroys the given event series.
     *
     * @section Events
     * @docs http://apidocs.apollohq.com/events
     * @example deleteSeries.php
     *
     * @param integer $event_id the event id
     */
    public function deleteCalSeries($event_id) {
        return $this->callServer("DELETE", "/cal-events/delete-series/$event_id");
    }
}

?>
